#include <cstdio>
#include <cstring>
#include <stdexcept>
#include <unistd.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <assert.h>
#include <sqlite3.h>

sqlite3* _sqlite3 = nullptr;

bool directory_exists(const char* fs_path)	
{
	struct stat ss;
	if (stat(fs_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFDIR;
}

bool file_exists(const char* file_path)	
{
	struct stat ss;
	if (stat(file_path, &ss) != 0)
		return false;
	else
		return (ss.st_mode & S_IFMT) == S_IFREG;
}

std::string get_cesta_db()
{
	std::string db_file = getenv("HOME");
	if (!directory_exists(db_file.c_str()))
		db_file = getpwuid(getuid())->pw_dir;
	if (!directory_exists(db_file.c_str()))
		throw std::runtime_error("Nepodařilo se zjistit domovský adresář.");
	db_file.append("/Dokumenty/test-stmt.db");
	return db_file;
}

void vytvorit_databazi()
{
	std::string str_file = get_cesta_db();
	if (file_exists(str_file.c_str()))
	{
		if (remove(str_file.c_str()) !=0)
			throw std::runtime_error("Chyba smazání souboru");
	}
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_exec(_sqlite3, "CREATE TABLE IF NOT EXISTS polozky (id INTEGER PRIMARY KEY,"
		" cislo INTEGER DEFAULT 0, nazev TEXT, cas INTEGER)", nullptr, nullptr, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void otevrit_databazi()
{
	std::string str_file = get_cesta_db();
	if (!file_exists(str_file.c_str()))
		throw std::runtime_error("Soubor databáze nebyl nalezen");
	int ires = sqlite3_open(str_file.c_str(), &_sqlite3);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void vypsat_data()
{
	assert(_sqlite3);
	sqlite3_stmt* stmt = nullptr;
	char sz_sql[255];
	strcpy(sz_sql, "SELECT cislo, nazev, cas FROM polozky");
	int ires = sqlite3_prepare_v2(_sqlite3, sz_sql,
		static_cast<int>(strlen(sz_sql)), &stmt, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	int istep;
	char* ptext;
	sqlite3_int64 cislo;
	struct tm* timeinfo;
	time_t cas;
	char sz_cas[16];
	istep = sqlite3_step(stmt);
	while (SQLITE_ROW == istep)
	{
		cislo = sqlite3_column_int64(stmt, 0);
		ptext = (char*)sqlite3_column_text(stmt, 1);
		cas = (time_t)sqlite3_column_int64(stmt, 2);
		timeinfo = localtime(&cas);
		strftime(sz_cas, sizeof(sz_cas), "%H:%M:%S", timeinfo);
		printf("%d, %s, %s\n", (int)cislo, ptext, sz_cas);
		istep = sqlite3_step(stmt);
		if (SQLITE_ERROR == istep)
			throw std::runtime_error(sqlite3_errstr(istep));
	}
	ires = sqlite3_reset(stmt);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	ires = sqlite3_finalize(stmt);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

void naplnit_nahodne(size_t pocet = 10)
{
	assert(_sqlite3);
	srand(time(nullptr));
	char sz_sql[255];
	char sz_text[255];
	int cislo;
	int ires;
	time_t cas;
	sqlite3_stmt* stmt = nullptr;
	strcpy(sz_sql, "INSERT INTO polozky (cislo, nazev, cas) VALUES (?,?,?)");
	int sqlres = sqlite3_prepare_v2(_sqlite3, sz_sql,
		static_cast<int>(strlen(sz_sql)), &stmt, nullptr);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
	for (size_t i = 0; i < pocet; i++)
	{
		cislo = rand() % 100 + 1;
		sprintf(sz_text, "%d - %X", cislo, cislo);
		time(&cas);
		ires = sqlite3_bind_int64(stmt, 1, (sqlite3_int64)cislo);
		if (ires != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(ires));
		ires = sqlite3_bind_text(stmt, 2, sz_text, (int)(strlen(sz_text)), SQLITE_STATIC);
		if (ires != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(ires));
		ires = sqlite3_bind_int64(stmt, 3, (sqlite3_int64)cas);
		if (ires != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(ires));
		ires = sqlite3_step(stmt);
		if ((sqlres != SQLITE_OK) && (sqlres != SQLITE_DONE))
			throw std::runtime_error(sqlite3_errstr(ires));
		ires = sqlite3_reset(stmt);
		if (ires != SQLITE_OK)
			throw std::runtime_error(sqlite3_errstr(ires));
		sleep(1);
	}
	ires = sqlite3_finalize(stmt);
	if (ires != SQLITE_OK)
		throw std::runtime_error(sqlite3_errstr(ires));
}

int main(int argc, char** argv)
{
	try
	{
		//vytvorit_databazi();
		//naplnit_nahodne();
		otevrit_databazi();
		vypsat_data();
		sqlite3_close(_sqlite3);
		printf("\nok\n");
		return EXIT_SUCCESS;
	}
	catch(const std::exception& e)
	{
		printf("Došlo k chybě: %s\n", e.what());
		if (_sqlite3)
			sqlite3_close(_sqlite3);
		return EXIT_FAILURE;
	}
}

